@@ -1,31 +1,33 @@
-"""
-Licensed to the Apache Software Foundation (ASF) under one
-or more contributor license agreements.  See the NOTICE file
-distributed with this work for additional information
-regarding copyright ownership.  The ASF licenses this file
-to you under the Apache License, Version 2.0 (the
-"License"); you may not use this file except in compliance
-with the License.  You may obtain a copy of the License at
-
-http://www.apache.org/licenses/LICENSE-2.0
-
-Unless required by applicable law or agreed to in writing,
-software distributed under the License is distributed on an
-"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
-KIND, either express or implied.  See the License for the
-specific language governing permissions and limitations
-under the License.
-"""
+#
+# Licensed to the Apache Software Foundation (ASF) under one
+# or more contributor license agreements.  See the NOTICE file
+# distributed with this work for additional information
+# regarding copyright ownership.  The ASF licenses this file
+# to you under the Apache License, Version 2.0 (the
+# "License"); you may not use this file except in compliance
+# with the License.  You may obtain a copy of the License at
+# 
+# http://www.apache.org/licenses/LICENSE-2.0
+# 
+# Unless required by applicable law or agreed to in writing,
+# software distributed under the License is distributed on an
+# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+# KIND, either express or implied.  See the License for the
+# specific language governing permissions and limitations
+# under the License.
+#
 import abc
 import base64
 
 import six
+
 try:
     import ujson as json
 except ImportError:
     import json
 
 from gremlin_python.driver import serializer, request
+from gremlin_python.driver.resultset import ResultSet
 
 __author__ = 'David M. Brown (davebshow@gmail.com)'
 
@@ -34,10 +36,11 @@ class GremlinServerError(Exception):
     def __init__(self, status):
         super(GremlinServerError, self).__init__("{0}: {1}".format(status["code"], status["message"]))
         self._status_attributes = status["attributes"]
+        self.status_code = status["code"]
 
     @property
     def status_attributes(self):
-        return self._status_attributes        
+        return self._status_attributes
 
 
 @six.add_metaclass(abc.ABCMeta)
@@ -74,11 +77,12 @@ class GremlinServerWSProtocol(AbstractBaseProtocol):
     def data_received(self, message, results_dict):
         # if Gremlin Server cuts off then we get a None for the message
         if message is None:
-            raise GremlinServerError("Server disconnected - please try to reconnect")
+            raise GremlinServerError({'code': 500, 
+                                      'message': 'Server disconnected - please try to reconnect', 'attributes': {}})
 
-        message = self._message_serializer.deserialize_message(json.loads(message.decode('utf-8')))
+        message = self._message_serializer.deserialize_message(message)
         request_id = message['requestId']
-        result_set = results_dict[request_id]
+        result_set = results_dict[request_id] if request_id in results_dict else ResultSet(None, None)
         status_code = message['status']['code']
         aggregate_to = message['result']['meta'].get('aggregateTo', 'list')
         data = message['result']['data']
@@ -92,7 +96,7 @@ class GremlinServerWSProtocol(AbstractBaseProtocol):
             self.write(request_id, request_message)
             data = self._transport.read()
             # Allow recursive call for auth
-            self.data_received(data, results_dict)
+            return self.data_received(data, results_dict)
         elif status_code == 204:
             result_set.stream.put_nowait([])
             del results_dict[request_id]
